Attribute VB_Name = "modWin32"
' Autor: F. Schler (frank@activevb.de)
' Datum: 03/2023

Option Explicit

' ----==== Const ====----
Public Const S_OK As Long = &H0&
Private Const CC_STDCALL As Long = &H4&
Public Const E_NOINTERFACE As Long = &H80004002
Private Const CLSCTX_INPROC_SERVER As Long = &H1&

Public Const IID_IUnknown As String = "{00000000-0000-0000-c000-000000000046}"
Private Const IID_IClosable As String = "{30d5a829-7fa4-4026-83bb-d75bae4ea99e}"
Private Const IID_IAsyncInfo As String = "{00000036-0000-0000-c000-000000000046}"
Private Const IID_IInitializeWithWindow As String = "{3e68d4bd-7135-4d10-8018-9fb6d9f33fa1}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IUnknown
    IUnknown_QueryInterface = 0
    IUnknown_Release = 2

    ' IInspectable
    IInspectable_GetIids = 3
    IInspectable_GetRuntimeClassName = 4
    IInspectable_GetTrustLevel = 5
    
    ' IAsyncInfo
    IAsyncInfo_GetStatus = 7
    IAsyncInfo_Close = 10

    ' IAsyncOperation
'    IAsyncOperation_PutCompleted = 6
'    IAsyncOperation_GetCompleted = 7
    IAsyncOperation_GetResults = 8
    
    ' IClosable
    IClosable_Close = 6

    ' IInitializeWithWindow
    IInitializeWithWindow_Initialize = 3

End Enum

Private Enum AsyncStatus
    Started = 0
    Completed = 1
    Canceled = 2
    Error = 3
End Enum

Private Enum TrustLevel
    BaseTrust = 0
    PartialTrust = 1
    FullTrust = 2
End Enum

Public Enum UserNotificationListenerAccessStatus
    UserNotificationListenerAccessStatus_Unspecified = 0
    UserNotificationListenerAccessStatus_Allowed = 1
    UserNotificationListenerAccessStatus_Denied = 2
End Enum

Public Enum NotificationKinds
    NotificationKinds_Unknown = 0
    NotificationKinds_Toast = 1
End Enum

Public Enum UserNotificationChangedKind
    UserNotificationChangedKind_Added = 0
    UserNotificationChangedKind_Removed = 1
End Enum

Public Enum AdaptiveNotificationContentKind
    AdaptiveNotificationContentKind_Text = 0
End Enum

' ----==== Types ====----
Public Type GUID
    data1 As Long
    data2 As Long
    data3 As Long
    data4 As Long
End Type
                
Public Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Public Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type
                
Public Type Windows_Foundation_DateTime
    UniversalTime As Currency
End Type
                
Public Type KeyValuePair_HSTRING_HSTRING
    key As String
    value As String
End Type

' ----==== Ole32.dll Declarations ====----
Private Declare Function CLSIDFromString Lib "ole32.dll" ( _
                         ByVal pstring As Long, _
                         ByRef pCLSID As GUID) As Long
                         
Private Declare Function StringFromGUID Lib "ole32.dll" _
                         Alias "StringFromGUID2" ( _
                         ByRef rguid As GUID, _
                         ByVal lpsz As Long, _
                         ByVal cchMax As Long) As Long
                         
Private Declare Function CoCreateInstance Lib "ole32.dll" ( _
                         ByVal rclsid As Long, _
                         ByVal pUnkOuter As Long, _
                         ByVal dwClsContext As Long, _
                         ByVal riid As Long, _
                         ByRef ppv As Long) As Long
                         
Private Declare Sub CoTaskMemFree Lib "ole32.dll" ( _
                    ByVal hMem As Long)
                         
' ----==== Oleaut32.dll Declarations ====----
Private Declare Function DispCallFunc Lib "Oleaut32.dll" ( _
                         ByVal pvInstance As Long, _
                         ByVal oVft As Long, _
                         ByVal cc As Long, _
                         ByVal vtReturn As VbVarType, _
                         ByVal cActuals As Long, _
                         ByVal prgvt As Any, _
                         ByVal prgpvarg As Any, _
                         ByRef pvargResult As Variant) As Long

' ----==== Kernel32.dll Declarations ====----
Public Declare Sub CopyMemory Lib "Kernel32.dll" _
                   Alias "RtlMoveMemory" ( _
                   ByRef pTo As Any, _
                   ByRef uFrom As Any, _
                   ByVal lSize As Long)

Private Declare Function lstrlenW Lib "Kernel32.dll" ( _
                         ByVal lpString As Long) As Long

Private Declare Function FileTimeToSystemTime Lib "Kernel32.dll" ( _
                         ByRef lpFileTime As FILETIME, _
                         ByRef lpSystemTime As SYSTEMTIME) As Long

Private Declare Function SystemTimeToFileTime Lib "Kernel32.dll" ( _
                         ByRef lpSystemTime As SYSTEMTIME, _
                         ByRef lpFileTime As FILETIME) As Long

Private Declare Function FileTimeToLocalFileTime Lib "Kernel32.dll" ( _
                         ByRef lpFileTime As FILETIME, _
                         ByRef lpLocalFileTime As FILETIME) As Long

Private Declare Function LocalFileTimeToFileTime Lib "Kernel32.dll" ( _
                         ByRef lpLocalFileTime As FILETIME, _
                         ByRef lpFileTime As FILETIME) As Long

' ----==== Combase.dll Declarations ====----
Private Declare Function WindowsCreateString Lib "Combase.dll" ( _
                         ByVal sourceString As Long, _
                         ByVal lenght As Long, _
                         ByRef hString As Long) As Long

Private Declare Function WindowsDeleteString Lib "Combase.dll" ( _
                         ByVal sourceString As Long) As Long

Private Declare Function WindowsGetStringRawBuffer Lib "Combase.dll" ( _
                         ByVal sourceString As Long, _
                         ByRef length As Long) As Long

Private Declare Function RoGetActivationFactory Lib "Combase.dll" ( _
                         ByVal activatableClassId As Long, _
                         ByRef riid As GUID, _
                         ByRef factory As Long) As Long

Private Declare Function RoActivateInstance Lib "Combase.dll" ( _
                         ByVal activatableClassId As Long, _
                         ByRef instance As Long) As Long

' ----==== WinRT Functions ====----
Public Function GetActivationFactory(ByVal ClassName As String, _
                                     ByVal iid As String, _
                                     ByRef pFactory As Long) As Boolean
    Dim hString As Long
    hString = CreateWindowsString(ClassName)
    If hString <> 0& Then
        If RoGetActivationFactory(hString, Str2Guid(iid), pFactory) = S_OK Then
            GetActivationFactory = True
        End If
        Call DeleteWindowsString(hString)
    End If
End Function

Public Function GetActivateInstance(ByVal ClassName As String, _
                                    ByVal iid As String, _
                                    ByRef pInstance As Long) As Boolean
    Dim hString As Long
    hString = CreateWindowsString(ClassName)
    If hString <> 0& Then
        Dim pIInspectable As Long
        If RoActivateInstance(hString, pIInspectable) = S_OK Then
            If pIInspectable <> 0& Then
                If QueryIfc(pIInspectable, iid, pInstance) Then
                    If pInstance <> 0& Then GetActivateInstance = True
                End If
                Call ReleaseIfc(pIInspectable)
            End If
        End If
        Call DeleteWindowsString(hString)
    End If
End Function

Public Function CreateWindowsString(ByVal value As String) As Long
    If value <> vbNullString Then
        Call WindowsCreateString(StrPtr(value), Len(value), CreateWindowsString)
    End If
End Function

Public Function GetWindowsString(ByVal hString As Long) As String
    If hString <> 0& Then
        GetWindowsString = Ptr2Str(WindowsGetStringRawBuffer(hString, 0&))
        Call DeleteWindowsString(hString)
    End If
End Function

Public Function DeleteWindowsString(ByRef hString As Long) As Boolean
    If hString <> 0& Then
        If WindowsDeleteString(hString) = S_OK Then
            hString = 0&
            DeleteWindowsString = True
        End If
    End If
End Function

Public Function InitWithWindow(ByVal pInterface As Long, _
                               ByVal ownerHwnd As Long) As Boolean
    If pInterface <> 0& And ownerHwnd <> 0& Then
        Dim pIInitializeWithWindow As Long
        If QueryIfc(pInterface, _
                    IID_IInitializeWithWindow, _
                    pIInitializeWithWindow) Then
            If InvokeIfc(pIInitializeWithWindow, _
                         IInitializeWithWindow_Initialize, _
                         ownerHwnd) = S_OK Then
                InitWithWindow = True
            End If
            Call ReleaseIfc(pIInitializeWithWindow)
        End If
    End If
End Function

Public Function Await(ByRef pInterface As Long, _
                      Optional ByVal WithResult As Boolean = True) As Boolean
    If pInterface <> 0& Then
        Dim pIAsyncInfo As Long
        If QueryIfc(pInterface, IID_IAsyncInfo, pIAsyncInfo) Then
            Dim eStatus As AsyncStatus
            Do
                If InvokeIfc(pIAsyncInfo, IAsyncInfo_GetStatus, VarPtr(eStatus)) = S_OK Then
                    DoEvents
                End If
            Loop While eStatus = Started
            If eStatus = Completed Then
                If WithResult Then
                    Dim pRet As Long
                    If InvokeIfc(pInterface, IAsyncOperation_GetResults, VarPtr(pRet)) = S_OK Then
                        If pRet <> 0& Then
                            Call ReleaseIfc(pInterface)
                            pInterface = pRet
                            Await = True
                        Else
                            Call ReleaseIfc(pInterface)
                        End If
                    Else
                        Call ReleaseIfc(pInterface)
                    End If
                Else
                    If InvokeIfc(pInterface, IAsyncOperation_GetResults) = S_OK Then
                        Await = True
                    Else
                        Call ReleaseIfc(pInterface)
                    End If
                End If
            Else
                Call ReleaseIfc(pInterface)
            End If
            Call InvokeIfc(pIAsyncInfo, IAsyncInfo_Close)
            Call ReleaseIfc(pIAsyncInfo)
        End If
    End If
End Function

Public Sub GetInspectableInfo(ByVal pInspectableInterface As Long, _
                              Optional ByVal interfaceName As String = vbNullString)
    If pInspectableInterface <> 0& Then
        If interfaceName <> vbNullString Then
            Debug.Print "Inspectable Info for: " & interfaceName
        End If
        Debug.Print "RuntimeClassName = " & GetRuntimeClassName(pInspectableInterface)
        Dim eTrustLevel As TrustLevel
        eTrustLevel = GetTrustLevel(pInspectableInterface)
        Select Case eTrustLevel
            Case TrustLevel.BaseTrust
                Debug.Print "TrustLevel = BaseTrust"
            Case TrustLevel.FullTrust
                Debug.Print "TrustLevel = FullTrust"
            Case TrustLevel.PartialTrust
                Debug.Print "TrustLevel = PartialTrust"
            Case Else
                Debug.Print "TrustLevel = UnknownTrust"
        End Select
        Dim tIIDs() As GUID
        Dim lngIIDsCount As Long
        lngIIDsCount = GetIIDs(pInspectableInterface, tIIDs)
        Debug.Print "AviableIIDsCount = " & CStr(lngIIDsCount)
        If lngIIDsCount > 0 Then
            Dim lngIIDsItem As Long
            For lngIIDsItem = 0 To lngIIDsCount - 1
                Debug.Print "    AviableIID " & CStr(lngIIDsItem + 1) & _
                            " = " & LCase$(Guid2Str(tIIDs(lngIIDsItem)))
            Next
        End If
        Debug.Print String(60, "-")
    End If
End Sub

Private Function GetIIDs(ByVal pInterface As Long, _
                         ByRef iids() As GUID) As Long
    If pInterface <> 0& Then
        Dim count As Long
        Dim pIIDs As Long
        If InvokeIfc(pInterface, IInspectable_GetIids, VarPtr(count), VarPtr(pIIDs)) = S_OK Then
            If count > 0& Then
                Dim bytes As Long
                Dim item As Long
                ReDim iids(count - 1)
                For item = 0 To count - 1
                    bytes = Len(iids(item))
                    Call CopyMemory(iids(item), ByVal pIIDs + (bytes * item), bytes)
                Next
                GetIIDs = count
            End If
        End If
    End If
End Function

Private Function GetRuntimeClassName(ByVal pInterface As Long) As String
    If pInterface <> 0& Then
        Dim hString As Long
        If InvokeIfc(pInterface, IInspectable_GetRuntimeClassName, VarPtr(hString)) = S_OK Then
            If hString <> 0& Then
                GetRuntimeClassName = GetWindowsString(hString)
            End If
        End If
    End If
End Function

Private Function GetTrustLevel(ByVal pInterface As Long) As TrustLevel
    If pInterface <> 0& Then
        Dim value As Long
        If InvokeIfc(pInterface, IInspectable_GetTrustLevel, VarPtr(value)) = S_OK Then
            GetTrustLevel = value
        End If
    End If
End Function

Public Sub DisposeIfc(ByRef pInterface As Long)
    If pInterface <> 0& Then
        Dim pIClosable As Long
        If InvokeIfc(pInterface, IUnknown_QueryInterface, _
                     VarPtr(Str2Guid(IID_IClosable)), _
                     VarPtr(pIClosable)) = S_OK Then
            Call InvokeIfc(pIClosable, IClosable_Close)
            Call ReleaseIfc(pIClosable)
            Call ReleaseIfc(pInterface)
            pInterface = 0&
        End If
    End If
End Sub

' ----==== Interface Functions ====----
Public Function CreateIfc(ByVal clsid As String, _
                          ByVal iid As String, _
                          ByRef pInterface As Long) As Boolean
    Dim Ret As Boolean
    If CoCreateInstance(VarPtr(Str2Guid(clsid)), _
                        0&, _
                        CLSCTX_INPROC_SERVER, _
                        VarPtr(Str2Guid(iid)), _
                        pInterface) = S_OK Then
        Ret = True
    End If
    CreateIfc = Ret
End Function

Public Function QueryIfc(ByVal pInterface As Long, _
                         ByVal iid As String, _
                         ByRef ppInterface As Long) As Boolean
    If pInterface <> 0& Then
        If InvokeIfc(pInterface, IUnknown_QueryInterface, _
                     VarPtr(Str2Guid(iid)), VarPtr(ppInterface)) = S_OK Then
            If ppInterface <> 0& Then QueryIfc = True
        End If
    End If
End Function

Public Sub ReleaseIfc(ByRef pInterface As Long)
    If pInterface <> 0& Then
        Call InvokeIfc(pInterface, _
                       IUnknown_Release)
        pInterface = 0&
    End If
End Sub

Private Function InvokeIfc(ByVal pInterface As Long, _
                           ByVal vtb As vtb_Interfaces, _
                           ParamArray var()) As Variant
    If pInterface <> 0& Then
        InvokeIfc = OleInvoke(pInterface, vtb, var)
    End If
End Function

Public Function OleInvoke(ByVal pInterface As Long, _
                          ByVal lngCmd As Long, _
                          ParamArray aParam()) As Variant
    If pInterface <> 0& Then
        If UBound(aParam) >= 0& Then
            Dim varParam As Variant
            varParam = aParam
            If IsArray(varParam) Then varParam = varParam(0)
            Dim lngCount As Long
            lngCount = UBound(varParam)
            Dim lngItem As Long
            Dim olePtr(10) As Long
            Dim oleTyp(10) As Integer
            For lngItem = 0& To lngCount
                oleTyp(lngItem) = VarType(varParam(lngItem))
                olePtr(lngItem) = VarPtr(varParam(lngItem))
            Next
        End If
        Dim varRet As Variant
        If DispCallFunc(pInterface, lngCmd * 4, CC_STDCALL, _
                        vbLong, lngItem, VarPtr(oleTyp(0)), _
                        VarPtr(olePtr(0)), varRet) <> S_OK Then
            Debug.Print "Fehler beim Aufrufen der Interface-Funktion!"
        End If

'        If varRet <> S_OK Then
'            Debug.Print "0x" & Hex$(varRet)
'        End If
    End If
    OleInvoke = varRet
End Function

' ----==== Helper Functions ====----
Public Function ProcPtr(ByVal ptr As Long) As Long
    ProcPtr = ptr
End Function

Public Function Str2Guid(ByVal strGUID As String) As GUID
    If Len(strGUID) = 38 Then
        Call CLSIDFromString(StrPtr(strGUID), Str2Guid)
    End If
End Function

Public Function Guid2Str(ByRef tGUID As GUID) As String
    Dim lngRet As Long
    Dim strGUID As String
    strGUID = String(40, 0)
    lngRet = StringFromGUID(tGUID, StrPtr(strGUID), Len(strGUID))
    If lngRet > 0& Then
        Guid2Str = Left$(strGUID, lngRet - 1)
    Else
        Guid2Str = vbNullString
    End If
End Function

Public Function Ptr2Str(ByVal lpStr As Long) As String
    If lpStr <> 0& Then
        Dim lngLen As Long
        Dim bytBuffer() As Byte
        lngLen = lstrlenW(lpStr) * 2
        If lngLen > 0 Then
            ReDim bytBuffer(lngLen - 1)
            Call CopyMemory(bytBuffer(0), ByVal lpStr, lngLen)
            Call CoTaskMemFree(lpStr)
            Ptr2Str = bytBuffer
        End If
    End If
End Function

Public Function DateTime2VBDate(ByRef value As Windows_Foundation_DateTime, _
                                Optional ByVal ToLocalFileTime As Boolean = True) As Date
    If value.UniversalTime <> CCur(0) Then
        Dim tFILETIME As FILETIME
        Dim tSYSTEMTIME As SYSTEMTIME
        Call CopyMemory(tFILETIME, value.UniversalTime, 8)
        If ToLocalFileTime Then
            Call FileTimeToLocalFileTime(tFILETIME, tFILETIME)
        End If
        If CBool(FileTimeToSystemTime(tFILETIME, tSYSTEMTIME)) Then
            With tSYSTEMTIME
                DateTime2VBDate = DateSerial(.wYear, .wMonth, .wDay) + TimeSerial(.wHour, .wMinute, .wSecond)
            End With
        End If
    End If
End Function

Public Function VBDate2DateTime(ByVal value As Date) As Windows_Foundation_DateTime
    Dim tFILETIME As FILETIME
    Dim tSYSTEMTIME As SYSTEMTIME
    With tSYSTEMTIME
        .wYear = Year(value)
        .wMonth = Month(value)
        .wDay = Day(value)
        .wHour = Hour(value)
        .wMinute = Minute(value)
        .wSecond = Second(value)
    End With
    If CBool(SystemTimeToFileTime(tSYSTEMTIME, tFILETIME)) Then
        If CBool(LocalFileTimeToFileTime(tFILETIME, tFILETIME)) Then
            Call CopyMemory(VBDate2DateTime.UniversalTime, tFILETIME, 8)
        End If
    End If
End Function

